@@ -2,7 +2,7 @@
 
 """
 
-CURRENT_VERSION_PACKAGE = "1.4.2"  # used when saving the config of a tokenizer
+CURRENT_VERSION_PACKAGE = "1.4.3"  # used when saving the config of a tokenizer
 
 # MIDI encodings default parameters, used when tokenizing a dataset and using tokens
 # These are the parameters from which a MIDI file will be tokenized
@@ -31,6 +31,10 @@ ADDITIONAL_TOKENS = {
     "time_signature_range": (8, 2),
 }  # (max_beat_res, max_bar_length_in_NOTE)
 
+# For MuMIDI, recommended range from the GM2 specs
+# note: we ignore the "Applause" at pitch 88 of the orchestra drum set, increase to 89 if you need it
+DRUM_PITCH_RANGE = range(27, 88)
+
 # Defaults values when writing new MIDI files
 TIME_DIVISION = 384  # 384 and 480 are convenient as divisible by 4, 8, 12, 16, 24, 32
 TEMPO = 120
