@@ -1,14 +1,11 @@
-""" MIDI encoding method, similar to Compound Word
-https://arxiv.org/abs/2101.02402
-
-"""
 
 from typing import List, Tuple, Dict, Optional, Union
+from pathlib import Path
 
 import numpy as np
 from miditoolkit import Instrument, Note, TempoChange
 
-from .midi_tokenizer_base import MIDITokenizer
+from .midi_tokenizer_base import MIDITokenizer, convert_tokens_tensors_to_list
 from .vocabulary import Vocabulary, Event
 from .utils import detect_chords
 from .constants import (
@@ -24,38 +21,44 @@ from .constants import (
 
 
 class CPWord(MIDITokenizer):
-    r"""MIDI encoding method, similar to Compound Word
-    https://arxiv.org/abs/2101.02402
-    Each compound token will be a list of the form:
-        (index. Token type)
-        0. Family
-        1. Bar/Position
-        2. Pitch
-        3. Velocity
-        4. Duration
-        (5. Program) optional, associated with notes (pitch/velocity/duration) or chords
-        (6. Chord) optional, chords occurring with position tokens
-        (7. Rest) optional, rest acting as a TimeShift token
-        (8. Tempo) optional, occurring with position tokens
-    This means a "compound token" can contain between 5 and 7 elements depending on
-    your encoding parameters (additional tokens).
-    (the choice of using indexes instead of dictionary with keys is to reduce the memory
-    and storage usage for saved token files)
+    r"""Introduced with the
+    `Compound Word Transformer (Hsiao et al.) <https://ojs.aaai.org/index.php/AAAI/article/view/16091>`_,
+    this tokenization is similar to :ref:`REMI` but uses embedding pooling operations to reduce
+    the overall sequence length: note tokens (*Pitch*, *Velocity* and *Duration*) are first
+    independently converted to embeddings which are then merged (pooled) into a single one.
+    Each compound token will be a list of the form (index: Token type):
+    * 0: Family
+    * 1: Bar/Position
+    * 2: Pitch
+    * 3: Velocity
+    * 4: Duration
+    * (+ Optional) Program: associated with notes (pitch/velocity/duration) or chords
+    * (+ Optional) Chord: chords occurring with position tokens
+    * (+ Optional) Rest: rest acting as a TimeShift token
+    * (+ Optional) Tempo: occurring with position tokens
 
-    :param pitch_range: range of used MIDI pitches
-    :param beat_res: beat resolutions, with the form:
+    The output hidden states of the model will then be fed to several output layers
+    (one per token type). This means that the training requires to add multiple losses.
+    For generation, the decoding implies sample from several distributions, which can be
+    very delicate. Hence, we do not recommend this tokenization for generation with small models.
+
+    :param pitch_range: range of MIDI pitches to use
+    :param beat_res: beat resolutions, as a dictionary:
             {(beat_x1, beat_x2): beat_res_1, (beat_x2, beat_x3): beat_res_2, ...}
-            The keys of the dict are tuples indicating a range of beats, ex 0 to 3 for the first bar
-            The values are the resolution, in samples per beat, of the given range, ex 8
+            The keys are tuples indicating a range of beats, ex 0 to 3 for the first bar, and
+            the values are the resolution to apply to the ranges, in samples per beat, ex 8
     :param nb_velocities: number of velocity bins
-    :param additional_tokens: specifies additional tokens (chords, time signature, rests, tempo...)
-    :param pad: will include a PAD token, used when training a model with batch of sequences of
-            unequal lengths, and usually at index 0 of the vocabulary. (default: True)
-    :param sos_eos: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary.
-            (default: False)
-    :param mask: will add a MASK token to the vocabulary (default: False)
-    :param sep: will add a SEP token to the vocabulary (default: False)
-    :param params: can be a path to the parameter (json encoded) file or a dictionary. (default: None)
+    :param additional_tokens: additional tokens (chords, time signature, rests, tempo...) to use,
+            to be given as a dictionary. (default: None is used)
+    :param pad: will add a special *PAD* token to the vocabulary, to use to pad sequences when
+            training a model with batches of different sequence lengths. (default: True)
+    :param sos_eos: adds special Start Of Sequence (*SOS*) and End Of Sequence (*EOS*) tokens
+            to the vocabulary. (default: False)
+    :param mask: will add a special *MASK* token to the vocabulary (default: False)
+    :param sep: will add a special *SEP* token to the vocabulary (default: False)
+    :param params: path to a tokenizer config file. This will override other arguments and
+            load the tokenizer based on the config file. This is particularly useful if the
+            tokenizer learned Byte Pair Encoding. (default: None)
     """
 
     def __init__(
@@ -68,7 +71,7 @@ class CPWord(MIDITokenizer):
         sos_eos: bool = False,
         mask: bool = False,
         sep: bool = False,
-        params=None,
+        params: Union[str, Path] = None,
     ):
         # Indexes of additional token types within a compound token
         additional_tokens["TimeSignature"] = False  # not compatible
@@ -128,7 +131,6 @@ class CPWord(MIDITokenizer):
         for note in track.notes:
             # Bar / Position / (Tempo) / (Rest)
             if note.start != previous_tick:
-
                 # (Rest)
                 if (
                     self.additional_tokens["Rest"]
@@ -297,7 +299,7 @@ class CPWord(MIDITokenizer):
         :return: The compound token as a list of integers
         """
         cp_token_template = [
-            Event(type_="Family", value="Metric", time=time, desc=desc),
+            Event(type="Family", value="Metric", time=time, desc=desc),
             self.vocab[1].event_to_token["Ignore_None"],
             self.vocab[2].event_to_token["Ignore_None"],
             self.vocab[3].event_to_token["Ignore_None"],
@@ -542,17 +544,15 @@ class CPWord(MIDITokenizer):
         self._add_special_tokens_to_types_graph(dic)
         return dic
 
-    def token_types_errors(
-        self, tokens: List[List[int]], consider_pad: bool = False
-    ) -> float:
-        r"""Checks if a sequence of tokens is constituted of good token types
+    @convert_tokens_tensors_to_list
+    def token_types_errors(self, tokens: List[List[int]]) -> float:
+        r"""Checks if a sequence of tokens is made of good token types
         successions and returns the error ratio (lower is better).
         The Pitch and Position values are also analyzed:
             - a position token cannot have a value <= to the current position (it would go back in time)
             - a pitch token should not be present if the same pitch is already played at the current position
 
         :param tokens: sequence of tokens to check
-        :param consider_pad: if True will continue the error detection after the first PAD token (default: False)
         :return: the error ratio (lower is better)
         """
 
@@ -582,9 +582,8 @@ class CPWord(MIDITokenizer):
         current_pos = -1
         current_pitches = []
 
-        def check(tok: List[int]):
-            nonlocal err, previous_type, current_pos, current_pitches
-            token_type, token_value = cp_token_type(tok)
+        for token in tokens[1:]:
+            token_type, token_value = cp_token_type(token)
             # Good token type
             if token_type in self.tokens_types_graph[previous_type]:
                 if token_type == "Bar":  # reset
@@ -606,12 +605,4 @@ class CPWord(MIDITokenizer):
                 err += 1
             previous_type = token_type
 
-        if consider_pad:
-            for token in tokens[1:]:
-                check(token)
-        else:
-            for token in tokens[1:]:
-                if previous_type == "PAD":
-                    break
-                check(token)
         return err / len(tokens)
