@@ -1,14 +1,11 @@
-"""MIDI-like encoding method similar to ???
-Music Transformer:
 
-"""
-
-from typing import List, Tuple, Dict, Optional
+from typing import List, Tuple, Dict, Optional, Union
+from pathlib import Path
 
 import numpy as np
 from miditoolkit import Instrument, Note, TempoChange
 
-from .midi_tokenizer_base import MIDITokenizer
+from .midi_tokenizer_base import MIDITokenizer, convert_tokens_tensors_to_list
 from .vocabulary import Vocabulary, Event
 from .utils import detect_chords
 from .constants import (
@@ -24,31 +21,32 @@ from .constants import (
 
 
 class MIDILike(MIDITokenizer):
-    r"""MIDI-Like encoding, used with Music Transformer or MT3
-    https://arxiv.org/abs/1808.03715
-    This strategy simply convert MIDI messages into distinct tokens.
-    The token types are then NoteOn, Velocity, NoteOff and TimeShift
-    (+ the additional token types of MidiTok if desired).
-    NOTE: as this encoding uses only "Time Shifts" events to move in the time, and only
-    from one note to another. Hence it is suitable to encode continuous sequences of
-    notes without long periods of silence. If your dataset contains music with long
-    pauses, you might handle them with an appropriate "time shift" dictionary
-    (which values are made from the beat_res dict) or with a different encoding.
+    r"""Introduced in `This time with feeling (Oore et al.) <https://arxiv.org/abs/1808.03715>`_
+    and later used with `Music Transformer (Huang et al.) <https://openreview.net/forum?id=rJe4ShAcF7>`_
+    and `MT3 (Gardner et al.) <https://openreview.net/forum?id=iMSjopcOn0p>`_,
+    this tokenization simply converts MIDI messages (*NoteOn*, *NoteOff*, *TimeShift*...)
+    as tokens, hence the name "MIDI-Like".
+    **Note:** as MIDI-Like uses *TimeShifts* events to move the time from note to
+    note, it could be unsuited for tracks with long pauses. In such case, the
+    maximum *TimeShift* value will be used.
 
-    :param pitch_range: range of used MIDI pitches
-    :param beat_res: beat resolutions, with the form:
+    :param pitch_range: range of MIDI pitches to use
+    :param beat_res: beat resolutions, as a dictionary:
             {(beat_x1, beat_x2): beat_res_1, (beat_x2, beat_x3): beat_res_2, ...}
-            The keys of the dict are tuples indicating a range of beats, ex 0 to 3 for the first bar
-            The values are the resolution, in samples per beat, of the given range, ex 8
+            The keys are tuples indicating a range of beats, ex 0 to 3 for the first bar, and
+            the values are the resolution to apply to the ranges, in samples per beat, ex 8
     :param nb_velocities: number of velocity bins
-    :param additional_tokens: specifies additional tokens (chords, time signature, rests, tempo...)
-    :param pad: will include a PAD token, used when training a model with batch of sequences of
-            unequal lengths, and usually at index 0 of the vocabulary. (default: True)
-    :param sos_eos: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary.
-            (default: False)
-    :param mask: will add a MASK token to the vocabulary (default: False)
-    :param sep: will add a SEP token to the vocabulary (default: False)
-    :param params: can be a path to the parameter (json encoded) file or a dictionary
+    :param additional_tokens: additional tokens (chords, time signature, rests, tempo...) to use,
+            to be given as a dictionary. (default: None is used)
+    :param pad: will add a special *PAD* token to the vocabulary, to use to pad sequences when
+            training a model with batches of different sequence lengths. (default: True)
+    :param sos_eos: adds special Start Of Sequence (*SOS*) and End Of Sequence (*EOS*) tokens
+            to the vocabulary. (default: False)
+    :param mask: will add a special *MASK* token to the vocabulary (default: False)
+    :param sep: will add a special *SEP* token to the vocabulary (default: False)
+    :param params: path to a tokenizer config file. This will override other arguments and
+            load the tokenizer based on the config file. This is particularly useful if the
+            tokenizer learned Byte Pair Encoding. (default: None)
     """
 
     def __init__(
@@ -61,7 +59,7 @@ class MIDILike(MIDITokenizer):
         sos_eos: bool = False,
         mask: bool = False,
         sep: bool = False,
-        params=None,
+        params: Union[str, Path] = None,
     ):
         additional_tokens["TimeSignature"] = False  # not compatible
         super().__init__(
@@ -101,12 +99,12 @@ class MIDILike(MIDITokenizer):
         for n, note in enumerate(track.notes):
             # Note On
             events.append(
-                Event(type_="NoteOn", value=note.pitch, time=note.start, desc=note.end)
+                Event(type="NoteOn", value=note.pitch, time=note.start, desc=note.end)
             )
             # Velocity
             events.append(
                 Event(
-                    type_="Velocity",
+                    type="Velocity",
                     value=note.velocity,
                     time=note.start,
                     desc=f"{note.velocity}",
@@ -114,14 +112,14 @@ class MIDILike(MIDITokenizer):
             )
             # Note Off
             events.append(
-                Event(type_="NoteOff", value=note.pitch, time=note.end, desc=note.end)
+                Event(type="NoteOff", value=note.pitch, time=note.end, desc=note.end)
             )
         # Adds tempo events if specified
         if self.additional_tokens["Tempo"]:
             for tempo_change in self.current_midi_metadata["tempo_changes"]:
                 events.append(
                     Event(
-                        type_="Tempo",
+                        type="Tempo",
                         value=tempo_change.tempo,
                         time=tempo_change.time,
                         desc=tempo_change.tempo,
@@ -135,7 +133,6 @@ class MIDILike(MIDITokenizer):
         previous_tick = 0
         previous_note_end = track.notes[0].start + 1
         for e, event in enumerate(events.copy()):
-
             # No time shift
             if event.time == previous_tick:
                 pass
@@ -157,7 +154,7 @@ class MIDILike(MIDITokenizer):
                 if rest_beat > 0:
                     events.append(
                         Event(
-                            type_="Rest",
+                            type="Rest",
                             value=f"{rest_beat}.0",
                             time=rest_tick,
                             desc=f"{rest_beat}.0",
@@ -173,7 +170,7 @@ class MIDILike(MIDITokenizer):
                     )
                     events.append(
                         Event(
-                            type_="Rest",
+                            type="Rest",
                             value=f"0.{rest_pos_temp}",
                             time=rest_tick,
                             desc=f"0.{rest_pos_temp}",
@@ -188,7 +185,7 @@ class MIDILike(MIDITokenizer):
                     index = np.argmin(np.abs(dur_bins - time_shift))
                     events.append(
                         Event(
-                            type_="TimeShift",
+                            type="TimeShift",
                             value=".".join(map(str, self.durations[index])),
                             time=previous_tick,
                             desc=f"{time_shift} ticks",
@@ -201,7 +198,7 @@ class MIDILike(MIDITokenizer):
                 index = np.argmin(np.abs(dur_bins - time_shift))
                 events.append(
                     Event(
-                        type_="TimeShift",
+                        type="TimeShift",
                         value=".".join(map(str, self.durations[index])),
                         time=previous_tick,
                         desc=f"{time_shift} ticks",
@@ -398,17 +395,15 @@ class MIDILike(MIDITokenizer):
         self._add_special_tokens_to_types_graph(dic)
         return dic
 
-    def token_types_errors(
-        self, tokens: List[int], consider_pad: bool = False
-    ) -> float:
-        r"""Checks if a sequence of tokens is constituted of good token types
+    @convert_tokens_tensors_to_list
+    def token_types_errors(self, tokens: List[int]) -> float:
+        r"""Checks if a sequence of tokens is made of good token types
         successions and returns the error ratio (lower is better).
         The Pitch and Position values are also analyzed:
             - a NoteOn token should not be present if the same pitch is already being played
             - a NoteOff token should not be present the note is not being played
 
         :param tokens: sequence of tokens to check
-        :param consider_pad: if True will continue the error detection after the first PAD token (default: False)
         :return: the error ratio (lower is better)
         """
         nb_tok_predicted = len(tokens)  # used to norm the score
@@ -456,8 +451,6 @@ class MIDILike(MIDITokenizer):
                         err += 1  # this pitch wasn't being played
                     else:
                         current_pitches.remove(int(events[i].value))
-                elif not consider_pad and events[i].type == "PAD":
-                    break
 
             # Bad token type
             else:
