@@ -1,7 +1,3 @@
-"""Modified version of Octuple with no Program (Track) tokens
-To use mainly for tasks handling a single track.
-
-"""
 
 from math import ceil
 from pathlib import Path, PurePath
@@ -10,7 +6,7 @@ from typing import List, Tuple, Dict, Optional, Union
 import numpy as np
 from miditoolkit import Instrument, Note, TempoChange
 
-from .midi_tokenizer_base import MIDITokenizer
+from .midi_tokenizer_base import MIDITokenizer, convert_tokens_tensors_to_list
 from .vocabulary import Vocabulary
 from .constants import (
     PITCH_RANGE,
@@ -24,23 +20,35 @@ from .constants import (
 
 
 class OctupleMono(MIDITokenizer):
-    r"""Modified version of Octuple with no Program (Track) tokens
-    To use mainly for tasks handling a single track.
+    r"""OctupleMono is similar to :ref:`Octuple`
+    (`MusicBert (Zeng et al.) <https://arxiv.org/abs/2106.05630>`_) but without the
+    *Program* token. OctupleMono is hence better suited for tasks with one track.
+    Each pooled token will be a list of the form (index: Token type):
+    * 0: Pitch
+    * 1: Velocity
+    * 2: Duration
+    * 3: Position
+    * 4: Bar
+    * (+ Optional) Tempo
+    * (+ Optional) TimeSignature
 
-    :param pitch_range: range of used MIDI pitches
-    :param beat_res: beat resolutions, with the form:
+    :param pitch_range: range of MIDI pitches to use
+    :param beat_res: beat resolutions, as a dictionary:
             {(beat_x1, beat_x2): beat_res_1, (beat_x2, beat_x3): beat_res_2, ...}
-            The keys of the dict are tuples indicating a range of beats, ex 0 to 3 for the first bar
-            The values are the resolution, in samples per beat, of the given range, ex 8
+            The keys are tuples indicating a range of beats, ex 0 to 3 for the first bar, and
+            the values are the resolution to apply to the ranges, in samples per beat, ex 8
     :param nb_velocities: number of velocity bins
-    :param additional_tokens: specifies additional tokens (time signature, tempo)
-    :param pad: will include a PAD token, used when training a model with batch of sequences of
-            unequal lengths, and usually at index 0 of the vocabulary. (default: True)
-    :param sos_eos: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary.
-            (default: False)
-    :param mask: will add a MASK token to the vocabulary (default: False)
-    :param sep: will add a SEP token to the vocabulary (default: False)
-    :param params: can be a path to the parameter (json encoded) file or a dictionary
+    :param additional_tokens: additional tokens (chords, time signature, rests, tempo...) to use,
+            to be given as a dictionary. (default: None is used)
+    :param pad: will add a special *PAD* token to the vocabulary, to use to pad sequences when
+            training a model with batches of different sequence lengths. (default: True)
+    :param sos_eos: adds special Start Of Sequence (*SOS*) and End Of Sequence (*EOS*) tokens
+            to the vocabulary. (default: False)
+    :param mask: will add a special *MASK* token to the vocabulary (default: False)
+    :param sep: will add a special *SEP* token to the vocabulary (default: False)
+    :param params: path to a tokenizer config file. This will override other arguments and
+            load the tokenizer based on the config file. This is particularly useful if the
+            tokenizer learned Byte Pair Encoding. (default: None)
     """
 
     def __init__(
@@ -53,7 +61,7 @@ class OctupleMono(MIDITokenizer):
         sos_eos: bool = False,
         mask: bool = False,
         sep: bool = False,
-        params=None,
+        params: Union[str, Path] = None,
     ):
         additional_tokens["Chord"] = False  # Incompatible additional token
         additional_tokens["Rest"] = False
@@ -83,16 +91,10 @@ class OctupleMono(MIDITokenizer):
     def save_params(
         self, out_path: Union[str, Path, PurePath], additional_attributes: Dict = None
     ):
-        r"""Overrides the parent class method to include max_bar_embedding.
-        Saves the config / base parameters of the tokenizer in a file.
-        Useful to keep track of how a dataset has been tokenized / encoded
-        It will also save the name of the class used, i.e. the encoding strategy.
-        NOTE: if you override this method, you should probably call it (super()) at the end
-            and use the additional_attributes argument.
-        NOTE 2: as json cant save tuples as keys, the beat ranges are saved as strings
-        with the form startingBeat_endingBeat (underscore separating these two values)
+        r"""Saves the config / parameters of the tokenizer in a json encoded file.
+        This can be useful to keep track of how a dataset has been tokenized.
 
-        :param out_path: output path to save the file
+        :param out_path: output path to save the file.
         :param additional_attributes: any additional information to store in the config file.
                 It can be used to override the default attributes saved in the parent method. (default: None)
         """
@@ -314,19 +316,17 @@ class OctupleMono(MIDITokenizer):
         """
         return {}  # not relevant for this encoding
 
-    def token_types_errors(
-        self, tokens: List[List[int]], consider_pad: bool = False
-    ) -> float:
-        r"""Checks if a sequence of tokens is constituted of good token values and
+    @convert_tokens_tensors_to_list
+    def token_types_errors(self, tokens: List[List[int]]) -> float:
+        r"""Checks if a sequence of tokens is made of good token values and
         returns the error ratio (lower is better).
-        The token types are always the same in Octuple so this methods only checks
+        The token types are always the same in Octuple so this method only checks
         if their values are correct:
             - a bar token value cannot be < to the current bar (it would go back in time)
             - same for positions
             - a pitch token should not be present if the same pitch is already played at the current position
 
         :param tokens: sequence of tokens to check
-        :param consider_pad: if True will continue the error detection after the first PAD token (default: False)
         :return: the error ratio (lower is better)
         """
         err = 0
@@ -334,10 +334,6 @@ class OctupleMono(MIDITokenizer):
         current_pitches = []
 
         for token in tokens:
-            if consider_pad and all(
-                token[i] == self.vocab[i]["PAD_None"] for i in range(len(token))
-            ):
-                break
             if any(
                 self.vocab[i][token].split("_")[1] == "None"
                 for i, token in enumerate(token)
