@@ -1,9 +1,6 @@
-"""MIDI encoding method, similar to the REMI introduced in the Pop Music Transformer paper
-https://arxiv.org/abs/2002.00212
-
-"""
 
 from typing import List, Tuple, Dict, Optional, Union
+from pathlib import Path
 
 import numpy as np
 from miditoolkit import Instrument, Note, TempoChange
@@ -24,23 +21,35 @@ from .constants import (
 
 
 class REMI(MIDITokenizer):
-    r"""MIDI encoding method, similar to the REMI introduced in the Pop Music Transformer paper
-    https://arxiv.org/abs/2002.00212
+    r"""REMI, standing for Revamped MIDI and introduced with the
+    `Pop Music Transformer (Huang and Yang) <https://dl.acm.org/doi/10.1145/3394171.3413671>`_,
+    is a tokenization that represents notes as successions of *Pitch*, *Velocity* and *Duration*
+    tokens, and time with *Bar* and *Position* tokens. A *Bar* token indicate that a new bar
+    is beginning, and *Position* the current position within the current bar. The number of
+    positions is determined by the ``beat_res`` argument, the maximum value will be used as
+    resolution.
+    **NOTE:** in the original paper, the tempo information is represented as the succession
+    of two token types: a *TempoClass* indicating if the tempo is fast or slow, and a
+    *TempoValue* indicating its value. MidiTok only uses one *Tempo* token for its value
+    (see :ref:`Additional tokens`).
 
-    :param pitch_range: range of used MIDI pitches
-    :param beat_res: beat resolutions, with the form:
+    :param pitch_range: range of MIDI pitches to use
+    :param beat_res: beat resolutions, as a dictionary:
             {(beat_x1, beat_x2): beat_res_1, (beat_x2, beat_x3): beat_res_2, ...}
-            The keys of the dict are tuples indicating a range of beats, ex 0 to 3 for the first bar
-            The values are the resolution, in samples per beat, of the given range, ex 8
+            The keys are tuples indicating a range of beats, ex 0 to 3 for the first bar, and
+            the values are the resolution to apply to the ranges, in samples per beat, ex 8
     :param nb_velocities: number of velocity bins
-    :param additional_tokens: specifies additional tokens (chords, time signature, rests, tempo...)
-    :param pad: will include a PAD token, used when training a model with batch of sequences of
-            unequal lengths, and usually at index 0 of the vocabulary. (default: True)
-    :param sos_eos: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary.
-            (default: False)
-    :param mask: will add a MASK token to the vocabulary (default: False)
-    :param sep: will add a SEP token to the vocabulary (default: False)
-    :param params: can be a path to the parameter (json encoded) file or a dictionary
+    :param additional_tokens: additional tokens (chords, time signature, rests, tempo...) to use,
+            to be given as a dictionary. (default: None is used)
+    :param pad: will add a special *PAD* token to the vocabulary, to use to pad sequences when
+            training a model with batches of different sequence lengths. (default: True)
+    :param sos_eos: adds special Start Of Sequence (*SOS*) and End Of Sequence (*EOS*) tokens
+            to the vocabulary. (default: False)
+    :param mask: will add a special *MASK* token to the vocabulary (default: False)
+    :param sep: will add a special *SEP* token to the vocabulary (default: False)
+    :param params: path to a tokenizer config file. This will override other arguments and
+            load the tokenizer based on the config file. This is particularly useful if the
+            tokenizer learned Byte Pair Encoding. (default: None)
     """
 
     def __init__(
@@ -53,7 +62,7 @@ class REMI(MIDITokenizer):
         sos_eos: bool = False,
         mask: bool = False,
         sep: bool = False,
-        params=None,
+        params: Union[str, Path] = None,
     ):
         additional_tokens["TimeSignature"] = False  # not compatible
         super().__init__(
@@ -102,7 +111,6 @@ class REMI(MIDITokenizer):
         ].tempo
         for note in track.notes:
             if note.start != previous_tick:
-
                 # (Rest)
                 if (
                     self.additional_tokens["Rest"]
@@ -120,7 +128,7 @@ class REMI(MIDITokenizer):
                     if rest_beat > 0:
                         events.append(
                             Event(
-                                type_="Rest",
+                                type="Rest",
                                 value=f"{rest_beat}.0",
                                 time=previous_note_end,
                                 desc=f"{rest_beat}.0",
@@ -136,7 +144,7 @@ class REMI(MIDITokenizer):
                         )
                         events.append(
                             Event(
-                                type_="Rest",
+                                type="Rest",
                                 value=f"0.{rest_pos_temp}",
                                 time=previous_note_end,
                                 desc=f"0.{rest_pos_temp}",
@@ -152,7 +160,7 @@ class REMI(MIDITokenizer):
                 for i in range(nb_new_bars):
                     events.append(
                         Event(
-                            type_="Bar",
+                            type="Bar",
                             value="None",
                             time=(current_bar + i + 1) * ticks_per_bar,
                             desc=0,
@@ -164,7 +172,7 @@ class REMI(MIDITokenizer):
                 pos_index = int((note.start % ticks_per_bar) / ticks_per_sample)
                 events.append(
                     Event(
-                        type_="Position",
+                        type="Position",
                         value=pos_index,
                         time=note.start,
                         desc=note.start,
@@ -191,7 +199,7 @@ class REMI(MIDITokenizer):
                                 break  # this tempo change is beyond the current time step, we break the loop
                     events.append(
                         Event(
-                            type_="Tempo",
+                            type="Tempo",
                             value=current_tempo,
                             time=note.start,
                             desc=note.start,
@@ -202,11 +210,11 @@ class REMI(MIDITokenizer):
 
             # Pitch / Velocity / Duration
             events.append(
-                Event(type_="Pitch", value=note.pitch, time=note.start, desc=note.pitch)
+                Event(type="Pitch", value=note.pitch, time=note.start, desc=note.pitch)
             )
             events.append(
                 Event(
-                    type_="Velocity",
+                    type="Velocity",
                     value=note.velocity,
                     time=note.start,
                     desc=f"{note.velocity}",
@@ -216,7 +224,7 @@ class REMI(MIDITokenizer):
             index = np.argmin(np.abs(dur_bins - duration))
             events.append(
                 Event(
-                    type_="Duration",
+                    type="Duration",
                     value=".".join(map(str, self.durations[index])),
                     time=note.start,
                     desc=f"{duration} ticks",
@@ -309,7 +317,9 @@ class REMI(MIDITokenizer):
                         previous_note_end = max(
                             previous_note_end, current_tick + duration
                         )
-                except IndexError:  # A well constituted sequence should not raise an exception
+                except (
+                    IndexError
+                ):  # A well constituted sequence should not raise an exception
                     pass  # However with generated sequences this can happen, or if the sequence isn't finished
 
         if len(tempo_changes) > 1:
@@ -414,64 +424,6 @@ class REMI(MIDITokenizer):
         self._add_special_tokens_to_types_graph(dic)
         return dic
 
-    def token_types_errors(
-        self, tokens: List[int], consider_pad: bool = False
-    ) -> float:
-        r"""Checks if a sequence of tokens is constituted of good token types
-        successions and returns the error ratio (lower is better).
-        The Pitch and Position values are also analyzed:
-            - a position token cannot have a value <= to the current position (it would go back in time)
-            - a pitch token should not be present if the same pitch is already played at the current position
-
-        :param tokens: sequence of tokens to check
-        :param consider_pad: if True will continue the error detection after the first PAD token (default: False)
-        :return: the error ratio (lower is better)
-        """
-        nb_tok_predicted = len(tokens)  # used to norm the score
-        tokens = self.decompose_bpe(tokens) if self.has_bpe else tokens
-
-        # Override from here
-
-        err = 0
-        previous_type = self.vocab.token_type(tokens[0])
-        current_pos = -1
-        current_pitches = []
-
-        def check(tok: int):
-            nonlocal err, previous_type, current_pos, current_pitches
-            token_type, token_value = self.vocab.token_to_event[tok].split("_")
-
-            # Good token type
-            if token_type in self.tokens_types_graph[previous_type]:
-                if token_type == "Bar":  # reset
-                    current_pos = -1
-                    current_pitches = []
-                elif token_type == "Pitch":
-                    if int(token_value) in current_pitches:
-                        err += 1  # pitch already played at current position
-                    else:
-                        current_pitches.append(int(token_value))
-                elif token_type == "Position":
-                    if int(token_value) <= current_pos and previous_type != "Rest":
-                        err += 1  # token position value <= to the current position
-                    else:
-                        current_pos = int(token_value)
-                        current_pitches = []
-            # Bad token type
-            else:
-                err += 1
-            previous_type = token_type
-
-        if consider_pad:
-            for token in tokens[1:]:
-                check(token)
-        else:
-            for token in tokens[1:]:
-                if previous_type == "PAD":
-                    break
-                check(token)
-        return err / nb_tok_predicted
-
     @staticmethod
     def _order(x: Event) -> int:
         r"""Helper function to sort events in the right order
