@@ -1,9 +1,6 @@
-"""Structured MIDI encoding method as using in the Piano Inpainting Application
-https://arxiv.org/abs/2107.05944
-
-"""
 
 from typing import List, Tuple, Dict, Optional, Union
+from pathlib import Path
 
 import numpy as np
 from miditoolkit import Instrument, Note, TempoChange
@@ -22,31 +19,33 @@ from .constants import (
 
 
 class Structured(MIDITokenizer):
-    r"""Structured MIDI encoding method as using in the Piano Inpainting Application
-    https://arxiv.org/abs/2107.05944
-    The token types follows the specific pattern:
-    Pitch -> Velocity -> Duration -> Time Shift -> back to Pitch ...
-    NOTE: this encoding uses only "Time Shifts" events to move in the time, and only
-    from one note to another. Hence it is suitable to encode continuous sequences of
-    notes without long periods of silence. If your dataset contains music with long
-    pauses, you might handle them with an appropriate "time shift" dictionary
-    (which values are made from the beat_res dict) or with a different encoding.
+    r"""Introduced with the `Piano Inpainting Application <https://arxiv.org/abs/2002.00212>`_,
+    it is similar to :ref:`TSD` but is based on a consistent token type successions.
+    Token types always follow the same pattern: *Pitch* -> *Velocity* -> *Duration* -> *TimeShift*.
+    The latter is set to 0 for simultaneous notes.
+    To keep this property, no additional token can be inserted in MidiTok's implementation,
+    except *Program* that can be added to the vocabulary and are up to you to use.
+    **Note:** as Structured uses *TimeShifts* events to move the time from note to
+    note, it could be unsuited for tracks with long pauses. In such case, the
+    maximum *TimeShift* value will be used.
 
-    :param pitch_range: range of used MIDI pitches
-    :param beat_res: beat resolutions, with the form:
+    :param pitch_range: range of MIDI pitches to use
+    :param beat_res: beat resolutions, as a dictionary:
             {(beat_x1, beat_x2): beat_res_1, (beat_x2, beat_x3): beat_res_2, ...}
-            The keys of the dict are tuples indicating a range of beats, ex 0 to 3 for the first bar
-            The values are the resolution, in samples per beat, of the given range, ex 8
+            The keys are tuples indicating a range of beats, ex 0 to 3 for the first bar, and
+            the values are the resolution to apply to the ranges, in samples per beat, ex 8
     :param nb_velocities: number of velocity bins
-    :param additional_tokens: specifies additional tokens (chords, time signature, rests, tempo...), only programs
-            are considered here.
-    :param pad: will include a PAD token, used when training a model with batch of sequences of
-            unequal lengths, and usually at index 0 of the vocabulary. (default: True)
-    :param sos_eos: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary.
-            (default: False)
-    :param mask: will add a MASK token to the vocabulary (default: False)
-    :param sep: will add a SEP token to the vocabulary (default: False)
-    :param params: can be a path to the parameter (json encoded) file or a dictionary
+    :param additional_tokens: additional tokens (chords, time signature, rests, tempo...) to use,
+            to be given as a dictionary. (default: None is used)
+    :param pad: will add a special *PAD* token to the vocabulary, to use to pad sequences when
+            training a model with batches of different sequence lengths. (default: True)
+    :param sos_eos: adds special Start Of Sequence (*SOS*) and End Of Sequence (*EOS*) tokens
+            to the vocabulary. (default: False)
+    :param mask: will add a special *MASK* token to the vocabulary (default: False)
+    :param sep: will add a special *SEP* token to the vocabulary (default: False)
+    :param params: path to a tokenizer config file. This will override other arguments and
+            load the tokenizer based on the config file. This is particularly useful if the
+            tokenizer learned Byte Pair Encoding. (default: None)
     """
 
     def __init__(
@@ -59,7 +58,7 @@ class Structured(MIDITokenizer):
         sos_eos: bool = False,
         mask: bool = False,
         sep: bool = False,
-        params=None,
+        params: Union[str, Path] = None,
     ):
         # No additional tokens
         additional_tokens["Chord"] = False  # Incompatible additional token
@@ -101,7 +100,7 @@ class Structured(MIDITokenizer):
             index = np.argmin(np.abs(dur_bins - time_shift))
             events.append(
                 Event(
-                    type_="TimeShift",
+                    type="TimeShift",
                     value=".".join(map(str, self.durations[index])),
                     time=0,
                     desc=f"{time_shift} ticks",
@@ -112,12 +111,12 @@ class Structured(MIDITokenizer):
         for n, note in enumerate(track.notes[:-1]):
             # Pitch
             events.append(
-                Event(type_="Pitch", value=note.pitch, time=note.start, desc=note.pitch)
+                Event(type="Pitch", value=note.pitch, time=note.start, desc=note.pitch)
             )
             # Velocity
             events.append(
                 Event(
-                    type_="Velocity",
+                    type="Velocity",
                     value=note.velocity,
                     time=note.start,
                     desc=f"{note.velocity}",
@@ -128,7 +127,7 @@ class Structured(MIDITokenizer):
             index = np.argmin(np.abs(dur_bins - duration))
             events.append(
                 Event(
-                    type_="Duration",
+                    type="Duration",
                     value=".".join(map(str, self.durations[index])),
                     time=note.start,
                     desc=f"{duration} ticks",
@@ -139,7 +138,7 @@ class Structured(MIDITokenizer):
             index = np.argmin(np.abs(dur_bins - time_shift))
             events.append(
                 Event(
-                    type_="TimeShift",
+                    type="TimeShift",
                     time=note.start,
                     desc=f"{time_shift} ticks",
                     value=".".join(map(str, self.durations[index]))
@@ -154,7 +153,7 @@ class Structured(MIDITokenizer):
         else:
             events.append(
                 Event(
-                    type_="Pitch",
+                    type="Pitch",
                     value=track.notes[-1].pitch,
                     time=track.notes[-1].start,
                     desc=track.notes[-1].pitch,
@@ -162,7 +161,7 @@ class Structured(MIDITokenizer):
             )
             events.append(
                 Event(
-                    type_="Velocity",
+                    type="Velocity",
                     value=track.notes[-1].velocity,
                     time=track.notes[-1].start,
                     desc=f"{track.notes[-1].velocity}",
@@ -172,7 +171,7 @@ class Structured(MIDITokenizer):
             index = np.argmin(np.abs(dur_bins - duration))
             events.append(
                 Event(
-                    type_="Duration",
+                    type="Duration",
                     value=".".join(map(str, self.durations[index])),
                     time=track.notes[-1].start,
                     desc=f"{duration} ticks",
@@ -286,53 +285,3 @@ class Structured(MIDITokenizer):
         }
         self._add_special_tokens_to_types_graph(dic)
         return dic
-
-    def token_types_errors(
-        self, tokens: List[int], consider_pad: bool = False
-    ) -> float:
-        r"""Checks if a sequence of tokens is constituted of good token types
-        successions and returns the error ratio (lower is better).
-        The Pitch values are also analyzed:
-            - a pitch token should not be present if the same pitch is already played at the time
-
-        :param tokens: sequence of tokens to check
-        :param consider_pad: if True will continue the error detection after the first PAD token (default: False)
-        :return: the error ratio (lower is better)
-        """
-        nb_tok_predicted = len(tokens)  # used to norm the score
-        tokens = self.decompose_bpe(tokens) if self.has_bpe else tokens
-
-        # Override from here
-
-        err = 0
-        previous_type = self.vocab.token_type(tokens[0])
-        current_pitches = []
-
-        def check(tok: int):
-            nonlocal err, previous_type, current_pitches
-            token_type, token_value = self.vocab.token_to_event[tok].split("_")
-
-            # Good token type
-            if token_type in self.tokens_types_graph[previous_type]:
-                if token_type == "Pitch":
-                    if int(token_value) in current_pitches:
-                        err += 1  # pitch already played at current position
-                    else:
-                        current_pitches.append(int(token_value))
-                elif token_type == "TimeShift":
-                    if self._token_duration_to_ticks(token_value, 48) > 0:
-                        current_pitches = []  # moving in time, list reset
-            # Bad token type
-            else:
-                err += 1
-            previous_type = token_type
-
-        if consider_pad:
-            for token in tokens[1:]:
-                check(token)
-        else:
-            for token in tokens[1:]:
-                if previous_type == "PAD":
-                    break
-                check(token)
-        return err / nb_tok_predicted
