@@ -1,8 +1,6 @@
-"""Similar to MIDI-Like but with Duration tokens instead of NoteOff
-
-"""
 
 from typing import List, Tuple, Dict, Optional, Union, Any
+from pathlib import Path
 
 import numpy as np
 from miditoolkit import Instrument, Note, TempoChange
@@ -23,23 +21,30 @@ from .constants import (
 
 
 class TSD(MIDITokenizer):
-    r"""TSD, for TimeShift Duration.
-    It is similar to MIDI-Like but with Duration tokens instead of NoteOff
+    r"""TSD, for Time Shift Duration, is similar to MIDI-Like :ref:`MIDI-Like`
+    but uses explicit *Duration* tokens to represent note durations, which have
+    showed `better results than with NoteOff tokens <https://arxiv.org/abs/2002.00212>`_.
+    **Note:** as TSD uses *TimeShifts* events to move the time from note to
+    note, it could be unsuited for tracks with long pauses. In such case, the
+    maximum *TimeShift* value will be used.
 
-    :param pitch_range: range of used MIDI pitches
-    :param beat_res: beat resolutions, with the form:
+    :param pitch_range: range of MIDI pitches to use
+    :param beat_res: beat resolutions, as a dictionary:
             {(beat_x1, beat_x2): beat_res_1, (beat_x2, beat_x3): beat_res_2, ...}
-            The keys of the dict are tuples indicating a range of beats, ex 0 to 3 for the first bar
-            The values are the resolution, in samples per beat, of the given range, ex 8
+            The keys are tuples indicating a range of beats, ex 0 to 3 for the first bar, and
+            the values are the resolution to apply to the ranges, in samples per beat, ex 8
     :param nb_velocities: number of velocity bins
-    :param additional_tokens: specifies additional tokens (chords, time signature, rests, tempo...)
-    :param pad: will include a PAD token, used when training a model with batch of sequences of
-            unequal lengths, and usually at index 0 of the vocabulary. (default: True)
-    :param sos_eos: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary.
-            (default: False)
-    :param mask: will add a MASK token to the vocabulary (default: False)
-    :param sep: will add a SEP token to the vocabulary (default: False)
-    :param params: can be a path to the parameter (json encoded) file or a dictionary
+    :param additional_tokens: additional tokens (chords, time signature, rests, tempo...) to use,
+            to be given as a dictionary. (default: None is used)
+    :param pad: will add a special *PAD* token to the vocabulary, to use to pad sequences when
+            training a model with batches of different sequence lengths. (default: True)
+    :param sos_eos: adds special Start Of Sequence (*SOS*) and End Of Sequence (*EOS*) tokens
+            to the vocabulary. (default: False)
+    :param mask: will add a special *MASK* token to the vocabulary (default: False)
+    :param sep: will add a special *SEP* token to the vocabulary (default: False)
+    :param params: path to a tokenizer config file. This will override other arguments and
+            load the tokenizer based on the config file. This is particularly useful if the
+            tokenizer learned Byte Pair Encoding. (default: None)
     """
 
     def __init__(
@@ -52,7 +57,7 @@ class TSD(MIDITokenizer):
         sos_eos: bool = False,
         mask: bool = False,
         sep: bool = False,
-        params=None,
+        params: Union[str, Path] = None,
     ):
         additional_tokens["TimeSignature"] = False  # not compatible
         super().__init__(
@@ -92,11 +97,11 @@ class TSD(MIDITokenizer):
         for n, note in enumerate(track.notes):
             # Note On / Velocity / Duration
             events.append(
-                Event(type_="Pitch", value=note.pitch, time=note.start, desc=note.end)
+                Event(type="Pitch", value=note.pitch, time=note.start, desc=note.end)
             )
             events.append(
                 Event(
-                    type_="Velocity",
+                    type="Velocity",
                     value=note.velocity,
                     time=note.start,
                     desc=f"{note.velocity}",
@@ -106,7 +111,7 @@ class TSD(MIDITokenizer):
             index = np.argmin(np.abs(dur_bins - duration))
             events.append(
                 Event(
-                    type_="Duration",
+                    type="Duration",
                     value=".".join(map(str, self.durations[index])),
                     time=note.start,
                     desc=f"{duration} ticks",
@@ -117,7 +122,7 @@ class TSD(MIDITokenizer):
             for tempo_change in self.current_midi_metadata["tempo_changes"]:
                 events.append(
                     Event(
-                        type_="Tempo",
+                        type="Tempo",
                         value=tempo_change.tempo,
                         time=tempo_change.time,
                         desc=tempo_change.tempo,
@@ -131,7 +136,6 @@ class TSD(MIDITokenizer):
         previous_tick = 0
         previous_note_end = track.notes[0].start + 1
         for e, event in enumerate(events.copy()):
-
             # No time shift
             if event.time == previous_tick:
                 pass
@@ -153,7 +157,7 @@ class TSD(MIDITokenizer):
                 if rest_beat > 0:
                     events.append(
                         Event(
-                            type_="Rest",
+                            type="Rest",
                             value=f"{rest_beat}.0",
                             time=rest_tick,
                             desc=f"{rest_beat}.0",
@@ -169,7 +173,7 @@ class TSD(MIDITokenizer):
                     )
                     events.append(
                         Event(
-                            type_="Rest",
+                            type="Rest",
                             value=f"0.{rest_pos_temp}",
                             time=rest_tick,
                             desc=f"0.{rest_pos_temp}",
@@ -184,7 +188,7 @@ class TSD(MIDITokenizer):
                     index = np.argmin(np.abs(dur_bins - time_shift))
                     events.append(
                         Event(
-                            type_="TimeShift",
+                            type="TimeShift",
                             value=".".join(map(str, self.durations[index])),
                             time=previous_tick,
                             desc=f"{time_shift} ticks",
@@ -197,7 +201,7 @@ class TSD(MIDITokenizer):
                 index = np.argmin(np.abs(dur_bins - time_shift))
                 events.append(
                     Event(
-                        type_="TimeShift",
+                        type="TimeShift",
                         value=".".join(map(str, self.durations[index])),
                         time=previous_tick,
                         desc=f"{time_shift} ticks",
@@ -363,57 +367,6 @@ class TSD(MIDITokenizer):
         self._add_special_tokens_to_types_graph(dic)
         return dic
 
-    def token_types_errors(
-        self, tokens: List[int], consider_pad: bool = False
-    ) -> float:
-        r"""Checks if a sequence of tokens is constituted of good token types
-        successions and returns the error ratio (lower is better).
-        The Pitch and Position values are also analyzed:
-            - a Pitch token should not be present if the same pitch is already being played
-
-        :param tokens: sequence of tokens to check
-        :param consider_pad: if True will continue the error detection after the first PAD token (default: False)
-        :return: the error ratio (lower is better)
-        """
-        nb_tok_predicted = len(tokens)  # used to norm the score
-        tokens = self.decompose_bpe(tokens) if self.has_bpe else tokens
-
-        # Override from here
-
-        err = 0
-        previous_type = self.vocab.token_type(tokens[0])
-        current_pitches = []
-        if previous_type == "Pitch":
-            current_pitches.append(int(self.vocab[tokens[0]].split("_")[1]))
-
-        def check(tok: int):
-            nonlocal err, previous_type, current_pitches
-            token_type, token_value = self.vocab.token_to_event[tok].split("_")
-            # Good token type
-            if token_type in self.tokens_types_graph[previous_type]:
-                if token_type == "Pitch":
-                    if int(token_value) in current_pitches:
-                        err += 1  # pitch already being played
-                    else:
-                        current_pitches.append(int(token_value))
-                elif token_type in ["TimeShift", "Rest"]:
-                    current_pitches = []  # moving in time, list reset
-            # Bad token type
-            else:
-                err += 1
-            previous_type = token_type
-
-        if consider_pad:
-            for token in tokens[1:]:
-                check(token)
-        else:
-            for token in tokens[1:]:
-                if previous_type == "PAD":
-                    break
-                check(token)
-
-        return err / nb_tok_predicted
-
     def token_types_errors_training(
         self, x_tokens: List[int], y_tokens: List[int]
     ) -> Tuple[Union[float, Any]]:
